/*
 * Copyright 2017 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.vms.call.lib;

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;

import static dk.vajhoej.vms.call.VMS.*; 

/**
 * Class LIBRTL_Util is a high level wrapper library for LIB$ functions.
 */
public class LIBRTL_Util extends LIBRTL {
	/**
	 * Call LIB$FIND_FILE and LIB$FIND_FILE_END to find all file names matching specification.
	 * @param filespec file name specification
	 * @param proc callback for each found matching file
	 * @throws UnsupportedEncodingException if file spec can not be decoded
	 */
	public static void LIB$FIND_FILE(String filespec, FilenameProcessor proc) throws UnsupportedEncodingException {
		FIND_FILE_CTX ctx = new FIND_FILE_CTX(filespec);
		for(;;) {
			Result<String> temp = ctx.LIB$FIND_FILE();
			if(!isSuccess(temp.getStatus())) break;
			proc.process(temp.getResult());
		}
		ctx.LIB$FIND_FILE_END();
	}
	/**
	 * Call LIB$FIND_FILE and LIB$FIND_FILE_END to find all file names matching specification.
	 * @param filespec file name specification
	 * @return Java List with matching file names 
	 * @throws UnsupportedEncodingException if file spec can not be decoded
	 */
	public static List<String> LIB$FIND_FILE(String filespec) throws UnsupportedEncodingException {
		final List<String> res = new ArrayList<String>();
		LIB$FIND_FILE(filespec, new FilenameProcessor() {
			public void process(String filename) {
				res.add(filename);
			}
		});
		return res;
	}
}
