/*
 * Copyright 2017 Arne Vajh�j.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.vms.call.script;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import dk.vajhoej.vms.call.lib.VMSCodes;

/**
 * Class VMSCodesNames converts between VMS items codes numeric and text form. 
 */
public class VMSCodesNames {
	private static Map<String,Map<String,Integer>> nam2num;
	private static Map<String,Map<Integer,String>> num2nam;
	static {
		nam2num = new HashMap<String,Map<String,Integer>>();
		num2nam = new HashMap<String,Map<Integer,String>>();
		try {
			for(Field  f : VMSCodes.class.getFields()) {
				String name = f.getName();
				if(name.contains("$") && f.getType().equals(int.class)) {
					int ixu = name.indexOf('_');
					if(ixu > 0) {
						int value = f.getInt(null);
						String prefix = name.substring(0, ixu);
						String rest = name.substring(ixu + 1);
						if(!nam2num.containsKey(prefix)) {
							nam2num.put(prefix,  new HashMap<String,Integer>());
						}
						nam2num.get(prefix).put(rest, value);
						if(!num2nam.containsKey(prefix)) {
							num2nam.put(prefix,  new HashMap<Integer,String>());
						}
						num2nam.get(prefix).put(value, rest);
					}
				}
			}
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		}
	}
	/**
	 * Convert code number to code name.
	 * @param prefix facility aka SS, LIB etc.
	 * @param number code number
	 * @return code name
	 */
	public static String numberToName(String prefix, int number) {
		return num2nam.get(prefix).get(number);
	}
	/**
	 * Convert code number to code name.
	 * @param prefix facility aka SS, LIB etc.
	 * @param number code numbers
	 * @return code names
	 */
	public static String[] numberToName(String prefix, int[] number) {
		 String[] res = new String[number.length];
		 for(int i = 0; i < res.length; i++) {
			 res[i] = num2nam.get(prefix).get(number[i]);
		 }
		 return res;
	}
	/**
	 * Convert code name to code number.
	 * @param prefix facility aka SS, LIB etc. 
	 * @param name code name
	 * @return code number
	 */
	public static int nameToNumber(String prefix, String name) {
		if(nam2num.containsKey(prefix) && nam2num.get(prefix).containsKey(name)) {
			return nam2num.get(prefix).get(name);
		} else {
			return 0;
		}
	}
	/**
	 * Convert code number to name.
	 * @param prefix facility aka SS, LIB etc.
	 * @param name code names
	 * @return code numbers
	 */
	public static int[] nameToNumber(String prefix, String[] name) {
		int[] res = new int[name.length];
		for(int i = 0; i < res.length; i++) {
			res[i] = nameToNumber(prefix, name[i]);
		}
		return res;
	}
	/**
	 * Get all code numbers.
	 * @param prefix facility aka SS, LIB etc.
	 * @return code numbers
	 */
	public static int[] all(String prefix) {
		int[] res = new int[nam2num.get(prefix).size()];
		int ix = 0;
		for(int num : nam2num.get(prefix).values()) {
			res[ix] = num;
			ix++;
		}
		return res;
	}
	/**
	 * Convert Map of code numbers to values to Map of code names to values.
	 * @param <T> type of values
	 * @param prefix facility aka SS, LIB etc.
	 * @param map Map of code numbers to values
	 * @return Map of code names to values
	 */
	public static <T> Map<String,T> numberToName(String prefix, Map<Integer,T> map) {
		Map<String,T> res = new HashMap<String,T>();
		Iterator<Map.Entry<Integer,T>> it = map.entrySet().iterator();
		while(it.hasNext()) {
			Map.Entry<Integer,T> ent = it.next();
			res.put(num2nam.get(prefix).get(ent.getKey()), ent.getValue());
		}
		return res;
	}
	/**
	 * Convert List of Maps of code numbers to values to List of Maps of code names to values.
	 * @param <T> type of values
	 * @param prefix facility aka SS, LIB etc.
	 * @param lst List of Maps of code numbers to values
	 * @return List of Maps of code names to values
	 */
	public static <T> List<Map<String,T>> numberToName(String prefix, List<Map<Integer,T>> lst) {
		List<Map<String,T>> res = new ArrayList<Map<String,T>>();
		for(Map<Integer,T> m : lst) {
			res.add(numberToName(prefix, m));
		}
		return res;
	}
}
