/*
 * Copyright 2017 Arne Vajh�j.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.vms.call.test;

import static org.junit.Assert.*;

import java.util.List;

import org.junit.Test;

import static dk.vajhoej.vms.call.lib.VMSCodes.*;
import static dk.vajhoej.vms.call.lib.VMSStatus.*;

import dk.vajhoej.vms.call.VMSLibrary.Result;
import dk.vajhoej.vms.call.lib.LIBRTL;
import dk.vajhoej.vms.call.lib.LIBRTL_Util;

public class TestLIBRTL extends BaseTest {
	@Test
	public void testFIND_FILE() {
        try {
        	List<String> lres = LIBRTL_Util.LIB$FIND_FILE("*.com");
        	assertVMSCall("FIND_FILE *.com", SS$_NORMAL, lres);
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testGETDVI() {
        try {
        	Result<String> sres = LIBRTL.LIB$GETDVI_AsString(DVI$_VOLNAM, "SYS$DISK");
            assertVMSCall("GETDVI VOLNAM SYS$DISK", sres.getStatus(), sres.getResult());
            Result<Integer> ires = LIBRTL.LIB$GETDVI_AsInt(DVI$_MAXBLOCK, "SYS$DISK");
            assertVMSCall("GETDVI MAXBLOCK SYS$DISK", ires.getStatus(), ires.getResult());
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testGETJPI() {
        try {
        	Result<Integer> ires = LIBRTL.LIB$GETJPI_AsInt(JPI$_PID, 0);
            assertVMSCall("GETJPI PID 0", ires.getStatus(), ires.getResult());
            int pid = ires.getResult();
            Result<String> sres;
            sres = LIBRTL.LIB$GETJPI_AsString(JPI$_PRCNAM, pid);
            assertVMSCall("GETJPI PRCNAM pid", sres.getStatus(), sres.getResult());
            sres = LIBRTL.LIB$GETJPI_AsString(JPI$_USERNAME, pid);
            assertVMSCall("GETJPI USERNAME pid", sres.getStatus(), sres.getResult());
            sres = LIBRTL.LIB$GETJPI_AsString(JPI$_IMAGNAME, pid);
            assertVMSCall("GETJPI IMAGNAME pid", sres.getStatus(), sres.getResult());
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testGETSYI() {
        try {
        	Result<Integer> ires = LIBRTL.LIB$GETSYI_AsInt(SYI$_CHANNELCNT);
            assertVMSCall("GETSYI CHANNELCNT", ires.getStatus(), ires.getResult());
            Result<String> sres = LIBRTL.LIB$GETSYI_AsString(SYI$_NODENAME);
            assertVMSCall("GETSYI NODENAME", sres.getStatus(), sres.getResult());
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testLOGICAL() {
        try {
        	Result<Void> setres = LIBRTL.LIB$SET_LOGICAL("XNAM", "XVAL");
        	assertVMSCall("SET LOGICAL", setres.getStatus());
        	Result<String> getres = LIBRTL.LIB$GET_LOGICAL("XNAM");
        	assertVMSCall("GET LOGICAL", getres.getStatus(), getres.getResult());
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testSYMBOL() {
        try {
        	Result<Void> setres = LIBRTL.LIB$SET_SYMBOL("XNAM", "XVAL");
        	assertVMSCall("SET SYMBOL", setres.getStatus());
        	Result<String> getres = LIBRTL.LIB$GET_SYMBOL("XNAM");
        	assertVMSCall("GET SYMBOL", getres.getStatus(), getres.getResult());
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testFOREIGN() {
        try {
        	Result<String> getres = LIBRTL.LIB$GET_FOREIGN();
        	assertVMSCall("GET FOREIGN", getres.getStatus(), getres.getResult());
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testCOMMON() {
        try {
        	Result<Void> putres = LIBRTL.LIB$PUT_COMMON("BLA BLA");
        	assertVMSCall("PUT COMMON", putres.getStatus());
        	Result<String> getres = LIBRTL.LIB$GET_COMMON();
        	assertVMSCall("GET COMMON", getres.getStatus(), getres.getResult());
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testGETMSG() {
        try {
        	Result<String> getres = LIBRTL.LIB$SYS_GETMSG(SS$_ABORT);
        	assertVMSCall("SYS GETMSG", getres.getStatus(), getres.getResult());
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testWAIT() {
        try {
        	long t1 = System.currentTimeMillis();
        	Result<Void> getres = LIBRTL.LIB$WAIT(3.0);
        	long t2 = System.currentTimeMillis();
        	assertVMSCall("WAIT\n", getres.getStatus(), (int)(t2 - t1 - 2999));
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
}
