/*
 * Copyright 2017 Arne Vajh�j.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.vms.call.test;

import java.util.List;
import java.util.Map;

import static org.junit.Assert.*;

import org.junit.Test;

import static dk.vajhoej.vms.call.lib.VMSCodes.*;
import static dk.vajhoej.vms.call.lib.VMSStatus.*;

import dk.vajhoej.vms.call.VMS.Word;
import dk.vajhoej.vms.call.VMSLibrary.Pair;
import dk.vajhoej.vms.call.VMSLibrary.Result;
import dk.vajhoej.vms.call.lib.SYS;
import dk.vajhoej.vms.call.lib.SYS_Util;

public class TestSYS extends BaseTest {
	@Test
	public void testPROCESS_SCAN() {
        try {
        	Result<Integer> res = SYS.SYS$PROCESS_SCAN(PSCAN$_USERNAME, "*");
        	assertVMSCall("PROCESS_SCAN USERNAME=*", res.getStatus());
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testGETJPIW() {
        try {
			Result<Map<Integer,Object>> res = SYS.SYS$GETJPIW(0, new int[] { JPI$_PID, JPI$_USERNAME, JPI$_PRCNAM, JPI$_CPUTIM });
			assertVMSCall("GETJPIW pid=0", res.getStatus(), res.getResult());
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testPROCESS_SCAN_GETJPIW() {
        try {
        	List<Map<Integer,Object>> res = SYS_Util.SYS$PROCESS_SCAN_and_SYS$GETJPIW(PSCAN$_USERNAME, "*", new int[] { JPI$_PID, JPI$_USERNAME, JPI$_PRCNAM, JPI$_CPUTIM });
			assertVMSCall("PROCESS_SCAN USERNAME=* GETJPIW res=%s\n", SS$_NORMAL, res);
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testGETDVIW() {
        try {
			Result<Map<Integer,Object>> res = SYS.SYS$GETDVIW("SYS$DISK", new int[] { DVI$_VOLNAM, DVI$_MAXBLOCK, DVI$_FREEBLOCKS });
			assertVMSCall("GETDVIW devnam=SYS$DISK", res.getStatus(), res.getResult());
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testGETUAI() {
        try {
			Result<Map<Integer,Object>> res = SYS.SYS$GETUAI("ARNE", new int[] { UAI$_OWNER, UAI$_DEFDEV, UAI$_DEFDIR, UAI$_UIC });
			assertVMSCall("GETUAI usrnam=ARNE", res.getStatus(), res.getResult());
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testIDTOASC_ASCTOID() {
        try {
        	int id = (Integer)SYS.SYS$GETJPIW(0, new int[] { JPI$_UIC }).getResult().get(JPI$_UIC);
        	Result<String> res = SYS.SYS$IDTOASC(id);
        	assertVMSCall("IDTOASC ID=me", res.getStatus(), res.getResult());
        	Map<Integer,String> res2 = SYS_Util.SYS$IDTOASC(true);
        	assertVMSCall("IDTOASC ID=*\n", SS$_NORMAL, res2);
        	Result<Integer> res3 = SYS.SYS$ASCTOID("SYSTEM");
        	assertVMSCall("ASCTOID name=SYSTEM", res3.getStatus(), res3.getResult());
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testASCTIM() {
        try {
			Result<String> res = SYS.SYS$ASCTIM();
			assertVMSCall("ASCTIM", res.getStatus(), res.getResult());
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testGETTIM() {
        try {
			Result<Long> res = SYS.SYS$GETTIM();
			assertVMSCall("GETTIM", res.getStatus(), res.getResult());
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testGETTIM_ASCTIM_BINTIM() {
        try {
			Result<Long> lres = SYS.SYS$GETTIM();
			assertVMSCall("GETTIM", lres.getStatus(), lres.getResult());
			Result<String> sres = SYS.SYS$ASCTIM(lres.getResult());
			assertVMSCall("ASCTIM", sres.getStatus(), sres.getResult());
			Result<Long> lres2 = SYS.SYS$BINTIM(sres.getResult());
			assertVMSCall("BINTIM", lres2.getStatus(), lres2.getResult());
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testSETPRV() {
        try {
			Result<Long> lres = SYS.SYS$SETPRV();
			assertVMSCall("SETPRV", lres.getStatus(), (lres.getResult() & PRV$M_TMPMBX));
			assertVMSCall("SETPRV", lres.getStatus(), (lres.getResult() & PRV$M_SYSPRV) == 0 ? 1 : 0);
			Result<Void> vres = SYS.SYS$SETPRV(true, PRV$M_SYSPRV, false);
			assertVMSCall("SETPRV", vres.getStatus());
			Result<Long> lres2 = SYS.SYS$SETPRV();
			assertVMSCall("SETPRV", lres2.getStatus(), lres2.getResult() & PRV$M_SYSPRV);
			Result<Void> vres2 = SYS.SYS$SETPRV(false, PRV$M_SYSPRV, false);
			assertVMSCall("SETPRV", vres2.getStatus());
			Result<Long> lres3 = SYS.SYS$SETPRV();
			assertVMSCall("SETPRV", lres3.getStatus(), (lres3.getResult() & PRV$M_SYSPRV) == 0 ? 1 : 0);
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testENQW_DEQ() {
        try {
			Result<Integer> ires = SYS.SYS$ENQW("Test", LCK$K_CRMODE);
			assertVMSCall("ENQW", ires.getStatus(), ires.getResult());
			Result<Void> vres;
			vres = SYS.SYS$ENQW(ires.getResult(), LCK$K_EXMODE);
			assertVMSCall("ENQ", vres.getStatus());
			vres = SYS.SYS$DEQ(ires.getResult());
			assertVMSCall("DEQ", vres.getStatus());
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testCREMBX_DELMBX() {
        try {
			Result<Word> wres;
			Result<Void> vres;
			wres = SYS.SYS$CREMBX(false, 1000);
			assertVMSCall("CREMBX temp", wres.getStatus(), wres.getResult().getValue());
			vres = SYS.SYS$DELMBX( wres.getResult());
			assertVMSCall("DELMBX temp", vres.getStatus());
			vres = SYS.SYS$SETPRV(true, PRV$M_PRMMBX, false);
			assertVMSCall("SETPRV enable", vres.getStatus());
			wres = SYS.SYS$CREMBX(true, 1000);
			assertVMSCall("CREMBX perm", wres.getStatus(), wres.getResult().getValue());
			vres = SYS.SYS$DELMBX( wres.getResult());
			assertVMSCall("DELMBX perm", vres.getStatus());
			vres = SYS.SYS$SETPRV(true, PRV$M_PRMMBX, false);
			assertVMSCall("SETPRV disable", vres.getStatus());
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testGETMSG() {
        try {
			Result<String> sres = SYS.SYS$GETMSG(SS$_ABORT);
			assertVMSCall("GETMSG", sres.getStatus(), sres.getResult());
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testCREPRC() {
        try {
			Result<Integer> ires = SYS.SYS$CREPRC("SYS$SYSTEM:LOGINOUT.EXE", "Z.COM", "Z.OUT");
			assertVMSCall("CREPRC", ires.getStatus(), ires.getResult());
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testGETQUIW() {
        try {
        	Result<Map<Integer,Object>> res = SYS.SYS$GETQUIW_Queue("SYS$BATCH", new int[] { QUI$_QUEUE_STATUS, QUI$_EXECUTING_JOB_COUNT });
        	assertVMSCall("GETQUIW queue=SYS$BATCH", res.getStatus(), res.getResult());
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testGETQUIW_Wildcard() {
        try {
        	List<Map<Integer,Object>> res = SYS_Util.SYS$GETQUIW_QueueJobs("SYS$BATCH", new int[] { QUI$_ENTRY_NUMBER, QUI$_USERNAME, QUI$_LOG_SPECIFICATION });
        	assertVMSCall("GETQUIW wildcard queue=SYS$BATCH", SS$_NORMAL, res);
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testSNDJBCW_Job() {
        try {
			Result<Integer> ires = SYS.SYS$SNDJBCW_Submit("SYS$BATCH", "ZZ.COM");
			assertVMSCall("SNDJBCW submit", ires.getStatus(), ires.getResult());
			Result<Void> vres = SYS.SYS$SNDJBCW_Delete(ires.getResult());
			assertVMSCall("SNDJBCW delete", vres.getStatus());
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testCRELNM_TRNLNM() {
        try {
			Result<Void> vres = SYS.SYS$CRELNM(0, "LNM$PROCESS", "ZLOGICAL", PSL$C_USER, new String[] {	"A", "BB" });
			assertVMSCall("CRELNM", vres.getStatus());
			Result<String> sres = SYS.SYS$TRNLNM("LNM$PROCESS", "ZLOGICAL", PSL$C_USER, 0);
			assertVMSCall("TRNLNM 0", sres.getStatus(), sres.getResult());
			sres = SYS.SYS$TRNLNM("LNM$PROCESS", "ZLOGICAL", PSL$C_USER, 1);
			assertVMSCall("TRNLNM 1", sres.getStatus(), sres.getResult());
			vres = SYS.SYS$CRELNM("LNM$PROCESS", "ZLOGICAL", "XXX");
			Result<Pair<String,Integer>> pres = SYS.SYS$TRNLNM_Attr("LNM$PROCESS", "ZLOGICAL");
			assertVMSCall("TRNLNM trans", pres.getStatus(), pres.getResult().getItem1());
			assertVMSCall("TRNLNM attr", pres.getStatus(), pres.getResult().getItem2());
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testGETSYIW() {
        try {
			Result<Map<Integer,Object>> res = SYS.SYS$GETSYIW(new int[] { SYI$_NODENAME, SYI$_ARCH_NAME, SYI$_AVAILCPU_CNT });
			assertVMSCall("GETSYIW", res.getStatus(), res.getResult());
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
}
